#!/bin/bash

function check_pid () {
    [ $# -eq 1 ] || {
        echo "Function usage: check_pid <pid>"
        exit 1
    }

    local PID=$1

    echo "$(ps ax | awk '{print $1}' | grep ${PID})"
}

# set this to 0 to leave processes running at the end of the test
CLEANUP=1

DATE_TIME_START=$(date +%Y%m%d_%H%M%S)
QUEUE_SIZE=$((1024*1024*800))
QUEUE_NAME="qpid_client_memory_leak_${DATE_TIME_START}"
MSG_NUM=100000
MSG_SIZE=1024
PMAP_LOG_DIR=${PWD}/pmap_logs_${DATE_TIME_START}
[ -d ${PMAP_LOG_DIR} ] || mkdir -p ${PMAP_LOG_DIR}
# timeout
TO=15
# Number of loops. Adjust this value to change the time the testrun lasts.
LOOPS_NUM=40
SPOUT=${PWD}/spout
DRAIN=${PWD}/drain

echo "starting test"
killall spout
killall drain
service qpidd restart

echo "creating queue"
qpid-config add queue ${QUEUE_NAME} --max-queue-size ${QUEUE_SIZE} --max-queue-count ${MSG_NUM} --limit-policy ring
[ $? -eq 0 ] || {
  echo "queue creation failed!"
  exit 1
}

echo "sending messages to the queue"
${SPOUT} -c 0 -s ${MSG_SIZE} ${QUEUE_NAME} &
SPOUT_PID=$!

sleep ${TO}

echo "starting to drain messages from queue"
${DRAIN} -c 0 ${QUEUE_NAME} > /dev/null &
DRAIN_PID=$!

while [ -n "$(check_pid ${DRAIN_PID})" ]; do
  CURR_PMAP_LOG="${PMAP_LOG_DIR}/pmap_${DRAIN_PID}_$(date +%Y%m%d%H%M%S).log"
  pmap -d ${DRAIN_PID} >> "${CURR_PMAP_LOG}"
  sleep ${TO}
done &

for ((i=1; i<=${LOOPS_NUM}; i++)); do
  if [ -z "$(check_pid ${DRAIN_PID})" ]; then
    echo "drain is not running, breaking the loop"
    break
  else
    CURR_PMAP_LOG="${PMAP_LOG_DIR}/pmap_${DRAIN_PID}_${i_loop}_$(date +%Y%m%d%H%M%S).log"
    pmap -d ${DRAIN_PID} >> "${CURR_PMAP_LOG}"
    sleep ${TO}
  fi
done

[ ${CLEANUP} -eq 0 ] || {
  echo "stopping processes"
  kill -9 ${SPOUT_PID}
  kill -9 ${DRAIN_PID}
  service qpidd stop
}

echo "processing pmap logs"
# client's memory at the beginning
INIT_MEM=$(for FILE in $(ls ${PMAP_LOG_DIR}); do tail -1 ${PMAP_LOG_DIR}/${FILE}; done | head -15 | tail -10 | awk '{print substr($2,1,length($2)-1)}' \
      | awk 'BEGIN{s=0}{s=s+($1+0)}END{printf "%.0f",(s / (0.0 +NR))}')

# client's memory at the end
END_MEM=$(for FILE in $(ls ${PMAP_LOG_DIR}); do tail -1 ${PMAP_LOG_DIR}/${FILE}; done | tail -10 | awk '{print substr($2,1,length($2)-1)}' \
      | awk 'BEGIN{s=0}{s=s+($1+0)}END{printf "%.0f",(s / (0.0 +NR))}')

echo "initial memory consumption: ${INIT_MEM}KB, memory consumption at the end: ${END_MEM}KB"

if [ ${END_MEM} -gt ${INIT_MEM} ]; then
  echo "ERROR: test failed!"
else
  echo "OK: test passed"
fi

echo "test finished"
